package com.winhc.bigdata.spark.udf

import com.winhc.bigdata.spark.utils.BaseUtil
import org.apache.commons.lang3.StringUtils
import org.apache.spark.sql.SparkSession

import scala.annotation.meta.getter
import org.json4s._
import org.json4s.jackson.JsonMethods._

/**
 * @Author: XuJiakai
 * @Date: 2020/7/10 13:49
 * @Description:
 */
trait BaseFunc {
  @(transient@getter) protected val spark: SparkSession
  private val pattern = "[^\\u4e00-\\u9fa5a-zA-Z \\(\\)（）.]+".r


  def cleanup(): Unit ={
    //清理特殊字符
    spark.udf.register("cleanup", (col: String) => {
      BaseUtil.cleanup(col)
    })
  }

  def tyc_split(): Unit = {
    spark.udf.register("tyc_split", (name: String) => {
      if (StringUtils.isEmpty(name)) {
        null
      } else {
        name.split("\t;\t")
      }
    })
  }

  def json_utils(): Unit = {
    spark.udf.register("get_json_kv", (key: String, value: String) => {
      if (StringUtils.isNotBlank(value)) {
        "\"" + key + "\":\"" + value + "\""
      } else {
        "\"" + key + "\":" + value
      }
    })
  }

  def json_add_kv(): Unit = {
    spark.udf.register("json_add_str", (json: String, addVal: String) => {
      if (StringUtils.isNotBlank(json)) {
        "{" + addVal + "," + json.substring(1)
      } else {
        "{" + addVal + "}"
      }
    })
  }

  def map_2_json(): Unit = {
    spark.udf.register("map_2_json", (map: Map[String, String]) => {
      compact(render(Extraction.decompose(map)(DefaultFormats)))
    })
  }

  def company_split(): Unit = {
    spark.udf.register("company_split", (name: String) => {
      if (StringUtils.isEmpty(name)) {
        Array("")
      } else {
        pattern.split(name)
      }
    })
  }


  def area_code(): Unit = {
    spark.udf.register("get_province_code", (name: String) => {
      if (StringUtils.isNotEmpty(name) && name.trim.length == 6) {
        name.trim.substring(0, 2)
      } else {
        null
      }
    })
    spark.udf.register("get_city_code", (name: String) => {
      if (StringUtils.isNotEmpty(name) && name.trim.length == 6) {
        name.trim.substring(2, 4)
      } else {
        null
      }
    })
    spark.udf.register("get_county_code", (name: String) => {
      if (StringUtils.isNotEmpty(name) && name.trim.length == 6) {
        name.trim.substring(4, 6)
      } else {
        null
      }
    })
  }
}
